from flask import request as flask_request

from flask_sieve.validator import Validator
from flask_sieve.exceptions import ValidationException


class FormRequest:
    def __init__(self, request=None):
        request = request or flask_request
        self._validator = Validator(rules=self.rules(), request=request)
        self._validator.set_custom_messages(self.messages())
        self._validator.set_custom_handlers(self.custom_handlers())

    def validate(self):
        if self._validator.fails():
            raise ValidationException(self._validator.messages())
        return True

    def register_rule_handler(self, handler, message, params_count=0):
        pass

    @staticmethod
    def messages():
        return {}

    @staticmethod
    def custom_handlers():
        return {}


class JsonRequest(FormRequest):
    def __init__(self, request=None):
        request = request or flask_request
        if not request.is_json:
            raise ValidationException({'request': 'Request must be valid JSON'})
        self._validator = Validator(rules=self.rules(), request=request)




import json
import unittest

from flask import Flask

from flask_sieve.validator import Validator, validate

class TestValidator(unittest.TestCase):
    def setUp(self):
        self._validator = Validator()

    def test_translates_validations(self):
        self.set_validator_params(
            rules={'email': ['required', 'email']},
            request={'email': 'invalid_email'},
        )
        self.assertTrue(self._validator.fails())
        self.assertIn('valid email', str(self._validator.messages()))

    def test_translates_validations_with_param(self):
        self.set_validator_params(
            rules={'name': ['required', 'string', 'min:6']},
            request={'name': 'joe'},
        )
        self.assertTrue(self._validator.fails())
        self.assertIn('6 characters', str(self._validator.messages()))

    def test_translates_validations_with_rest_params(self):
        self.set_validator_params(
            rules={'name': ['required_unless:email,a@b.com,c@d.com']},
            request={'email': 'e@f.com'}
        )
        self.assertTrue(self._validator.fails())
        self.assertIn('required unless', str(self._validator.messages()))

    def test_translates_validations_with_all_params(self):
        self.set_validator_params(
            rules={'name': ['required_with:email,age']},
            request={'email': 'a@b.com', 'age': 2}
        )
        self.assertTrue(self._validator.fails())
        self.assertIn('required when', str(self._validator.messages()))

    def test_handles_different_types_of_requests(self):
        class MockMultiDict:
            def to_dict(self, flat=None):
                return {'field': 1}

        class MockRequest:
            def __init__(self):
                self.is_json = True
                self.form = self.files = MockMultiDict()
                self.json = self.form.to_dict()


        request = MockRequest()
        validator = Validator(request=request, rules={'field': ['numeric']})
        self.assertTrue(validator.passes())
        request.is_json = False
        validator.set_request(request)
        self.assertTrue(validator.passes())


    def test_validate_decorator(self):
        class FailingRequest:
            def validate(self):
                raise ValueError()
        @validate(FailingRequest)
        def failing_request_endpoint():
            pass
        with self.assertRaises(ValueError):
            failing_request_endpoint()

        class PassingRequest:
            def validate(self):
                pass
        @validate(PassingRequest)
        def passing_request_endpoint():
            return True
        self.assertTrue(passing_request_endpoint())

    def test_translates_validations_with_custom_messages(self):
        self.set_validator_params(
            rules={'email': ['required', 'email']},
            request={'email': ''}
        )
        self._validator.set_custom_messages({
            'email.required': 'Kindly provide the email',
            'email.email': 'Whoa! That is not valid',
        })
        self.assertTrue(self._validator.fails())
        self.assertDictEqual({
            'email': [
                'Kindly provide the email',
                'Whoa! That is not valid'
            ]
        }, self._validator.messages())

    def test_translates_validations_with_custom_handler(self):
        def validate_odd(value, **kwargs):
            return int(value) % 2
        self._validator.register_rule_handler(
            handler=validate_odd,
            message='This number must be odd.',
            params_count=0
        )
        self.set_validator_params(
            rules={'number': ['odd']},
            request={'number': 4}
        )
        self.assertTrue(self._validator.fails())
        self.assertDictEqual({
            'number': [
                'This number must be odd.'
            ]
        }, self._validator.messages())
        self.set_validator_params(
            rules={'number': ['odd']},
            request={'number': 3}
        )
        self.assertTrue(self._validator.passes())

    def test_cannot_set_custom_handler_without_validate_keyword(self):
        def method_odd(value, **kwargs):
            return int(value) % 2
        with self.assertRaises(ValueError):
            self._validator.register_rule_handler(
                handler=method_odd,
                message='This number must be odd.',
                params_count=0
            )

    def set_validator_params(self, rules=None, request=None):
        rules = rules or {}
        request = request or {}
        self._validator.set_rules(rules)
        self._validator.set_request(request)




from functools import wraps

from flask import request as flask_request

from flask_sieve.parser import Parser
from flask_sieve.translator import Translator
from flask_sieve.rules_processor import RulesProcessor


class Validator:
    def __init__(self, rules=None, request=None, custom_handlers=None):
        self._parser = Parser()
        self._translator = Translator()
        self._processor = RulesProcessor()
        self._rules = rules or {}
        self._custom_handlers = custom_handlers or {}
        self._request = self._parse_request(request or {})

    def set_rules(self, rules):
        self._rules = rules

    def set_request(self, request):
        self._request = self._parse_request(request or {})

    def set_custom_messages(self, messages):
        self._translator.set_custom_messages(messages)

    def set_custom_handlers(self, handlers):
        for handler in handlers:
            self.register_error_handler(**handler)

    def register_rule_handler(self, handler, message, params_count=0):
        if not handler.__name__.startswith('validate_'):
            raise ValueError(
                'Rule handlers must start with "validate_" name, %s provided'
                % (handler.__name__)
            )
        self._processor.register_rule_handler(
            handler=handler,
            message=message,
            params_count=params_count
        )
        handler_messages = {}
        for handler_name, handler_dict in self._processor.custom_handlers().items():
            handler_messages[handler_name[len('validate_'):]] = handler_dict['message']
        self._translator.set_handler_messages(handler_messages)

    def fails(self):
        return not self.passes()

    def passes(self):
        self._parser.set_rules(self._rules)
        self._processor.set_rules(self._parser.parsed_rules())
        self._processor.set_request(self._request)
        return self._processor.passes()

    def messages(self):
        self._translator.set_validations(self._processor.validations())
        return self._translator.translated_errors()

    @staticmethod
    def _parse_request(request):
        if isinstance(request, dict):
            return request
        # instance of flask.request ...
        if request.is_json:
            return request.json
        dict_request = {}
        dict_request.update(request.form.to_dict(flat=True))
        dict_request.update(request.files.to_dict(flat=True))
        return dict_request


def validate(Request):
    def decorator(fn):
        @wraps(fn)
        def wrapper(*args, **kwargs):
            req = Request()
            req.validate()
            return fn(*args, **kwargs)
        return wrapper
    return decorator



import unittest

from flask import Flask

from flask_sieve.exceptions import ValidationException, register_error_handler


class TestErrorHandler(unittest.TestCase):
    def test_error_handler(self):
        app = Flask(__name__)
        register_error_handler(app)
        self.assertIn(ValidationException, app.error_handler_spec[None][None])
        errors = {'field': 'Test error'}

        with app.app_context():
            response, status = app.error_handler_spec[None][None][ValidationException](
                ValidationException(errors)
            )
        self.assertEqual(400, status)
        self.assertIn('Test error', str(response.get_json()))



class Parser:
    def __init__(self, rules=None):
        self._rules = rules or {}

    def set_rules(self, rules):
        self._rules = rules

    def parsed_rules(self):
        parsed_rules = {}
        for attribute, rules in self._rules.items():
            attribute_rules = []
            for rule in rules:
                rule_name = rule_params = None
                if rule.startswith('regex'):
                    rule_name = 'regex'
                    rule_params = [rule[len('regex:'):]]
                elif rule.startswith('not_regex'):
                    rule_name = 'not_regex'
                    rule_params = [rule[len('not_regex:'):]]
                elif ':' in rule:
                    rule_name, rule_params = rule.split(':')
                    rule_params = rule_params.split(',')
                else:
                    rule_name = rule
                    rule_params = []

                attribute_rules.append({
                    'name': rule_name,
                    'params': rule_params,
                })

            parsed_rules[attribute] = attribute_rules
        return parsed_rules



from __future__ import absolute_import
import os
import re
import sys
import ast
import json
import pytz
import operator
import requests

from PIL import Image
from dateutil.parser import parse as dateparse
from werkzeug.datastructures import FileStorage


class RulesProcessor:
    def __init__(self, app=None, rules=None, request=None):
        self._app = app
        self._rules = rules or {}
        self._request = request or {}
        self._custom_handlers = {}
        self._attributes_validations = {}

    def validations(self):
        return self._attributes_validations

    def fails(self):
        return not self.passes()

    def custom_handlers(self):
        return self._custom_handlers

    def passes(self):
        passes = True
        self._attributes_validations = {}
        for attribute, rules in self._rules.items():
            should_bail = self._has_rule(rules, 'bail')
            nullable = self._has_rule(rules, 'nullable')
            validations = []
            for rule in rules:
                handler = self._get_rule_handler(rule['name'])
                value = self._attribute_value(attribute)
                attr_type = self._get_type(value, rules)
                is_valid = handler(
                    value=value,
                    attribute=attribute,
                    params=rule['params'],
                    nullable=nullable,
                    rules=rules
                )
                validations.append({
                    'attribute': attribute,
                    'rule': rule['name'],
                    'is_valid': is_valid,
                    'attribute_type': attr_type,
                    'params': rule['params'],
                })
                if not is_valid:
                    passes = False
                    if should_bail:
                        self._attributes_validations[attribute] = validations
                        return False
            self._attributes_validations[attribute] = validations
        return passes

    def set_rules(self, rules):
        self._rules = rules

    def set_request(self, request):
        self._request = request

    def register_rule_handler(self, handler, message, params_count=0):
        # add a params count check wrapper
        def checked_handler(*args, **kwargs):
            self._assert_params_size(
                size=params_count,
                params=kwargs['params'],
                rule=('custom rule %s' % (handler.__name__,))
            )
            return handler(*args, **kwargs)

        self._custom_handlers[handler.__name__] = {
            'handler': checked_handler,
            'message': message or ('%s check failed' % (handler.__name__,))
        }

    @staticmethod
    def validate_accepted(value, **kwargs):
        return value in [ 1, '1', 'true', 'yes', 'on', True]

    def validate_active_url(self, value, **kwargs):
        return self._can_call_with_method(requests.options, value)

    def validate_after(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='after')
        return self._compare_dates(value, params[0], operator.gt)

    def validate_after_or_equal(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='after_or_equal')
        return self._compare_dates(value, params[0], operator.ge)

    @staticmethod
    def validate_alpha(value, **kwargs):
        if not value:
            return False
        value = str(value).replace(' ', '')
        return value.isalpha()

    @staticmethod
    def validate_alpha_dash(value, **kwargs):
        if not value:
            return False
        value = str(value)
        acceptables = [ ' ', '-', '_' ]
        for acceptable in acceptables:
            value = value.replace(acceptable, '')
        return value.isalpha()

    @staticmethod
    def validate_alpha_num(value, **kwargs):
        if not value:
            return False
        value = str(value).replace(' ', '')
        return value.isalnum()

    @staticmethod
    def validate_array(value, **kwargs):
        try:
            return isinstance(ast.literal_eval(str(value)), list)
        except (ValueError, SyntaxError):
            return False

    @staticmethod
    def validate_bail(**kwargs):
        return True

    def validate_before(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='before')
        return self._compare_dates(value, params[0], operator.lt)

    def validate_before_or_equal(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='before_or_equal')
        return self._compare_dates(value, params[0], operator.le)

    def validate_between(self, value, params, **kwargs):
        self._assert_params_size(size=2, params=params, rule='between')
        value = self._get_size(value)
        lower = self._get_size(params[0])
        upper = self._get_size(params[1])
        return lower <= value and value <= upper

    @staticmethod
    def validate_boolean(value, **kwargs):
        return value in [True, False, 1, 0, '0', '1']

    def validate_confirmed(self, value, attribute, **kwargs):
        return value == self._attribute_value(attribute + '_confirmation')

    def validate_date(self, value, **kwargs):
        return self._can_call_with_method(dateparse, value)

    def validate_date_equals(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='date_equals')
        return self._compare_dates(value, params[0], operator.eq)

    def validate_different(self, value, attribute, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='different')
        return value != self._attribute_value(params[0])

    def validate_digits(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='digits')
        self._assert_with_method(int, params[0])
        size = int(params[0])
        is_numeric = self.validate_numeric(value)
        return is_numeric and len(str(value).replace('.', '')) == size

    def validate_digits_between(self, value, params, **kwargs):
        self._assert_params_size(size=2, params=params, rule='digits_between')
        self._assert_with_method(int, params[0])
        self._assert_with_method(int, params[1])
        lower = int(params[0])
        upper = int(params[1])
        is_numeric = self.validate_numeric(value)
        value_len = len(str(value).replace('.', ''))
        return is_numeric and lower <= value_len and value_len <= upper

    def validate_dimensions(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='dimensions')
        if not self.validate_file(value):
            return False
        try:
            image = Image.open(value.filename)
            w, h = image.size
            image.close()
            return ('%dx%d' % (w, h)) == params[0]
        except Exception:
            return False

    @staticmethod
    def validate_distinct(value, **kwargs):
        try:
            lst = ast.literal_eval(str(value))
            if not isinstance(lst, list):
                return False
            return len(set(lst)) == len(lst)
        except (ValueError, SyntaxError):
            return False

    @staticmethod
    def validate_email(value, **kwargs):
        pattern = re.compile("""
            ^
            [a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]
            +@[a-zA-Z0-9]
            (?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?
            (?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*
            $
        """, re.VERBOSE | re.IGNORECASE | re.DOTALL)
        return pattern.match(str(value)) is not None

    @staticmethod
    def validate_exists(value, **kwargs):
        return False

    def validate_extension(self, value, params, **kwargs):
        if not self.validate_file(value):
            return False
        self._assert_params_size(size=1, params=params, rule='extension')
        return value.filename.split('.')[-1].lower() == params[0]

    @staticmethod
    def validate_file(value, **kwargs):
        return isinstance(value, FileStorage)

    def validate_filled(self, value, attribute, nullable, **kwargs):
        if self.validate_present(attribute):
            return self.validate_required(value, attribute, nullable)
        return True

    def validate_gt(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='gt')
        value = self._get_size(value)
        upper = self._get_size(self._attribute_value(params[0]))
        return value > upper

    def validate_gte(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='gte')
        value = self._get_size(value)
        upper = self._get_size(self._attribute_value(params[0]))
        return value >= upper

    def validate_image(self, value, **kwargs):
        if not self.validate_file(value):
            return False
        ext = value.filename.split('.')[-1]
        return ext in ['jpg', 'jpeg', 'gif', 'png', 'bmp', 'svg', 'tiff', 'tif']

    @staticmethod
    def validate_in(value, params, **kwargs):
        return value in params

    def validate_in_array(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='in_array')
        other_value = self._attribute_value(params[0])
        try:
            lst = ast.literal_eval(str(other_value))
            return value in lst
        except (ValueError, SyntaxError):
            return False

    @staticmethod
    def validate_integer(value, **kwargs):
        return str(value).isdigit()

    def validate_ip(self, value, **kwargs):
        return self.validate_ipv4(value) or self.validate_ipv6(value)

    @staticmethod
    def validate_ipv6(value, **kwargs):
        # S/0: question 319279
        pattern = re.compile(r"""
            ^
            \s*                         # Leading whitespace
            (?!.*::.*::)                # Only a single whildcard allowed
            (?:(?!:)|:(?=:))            # Colon iff it would be part of a wildcard
            (?:                         # Repeat 6 times:
                [0-9a-f]{0,4}           #   A group of at most four hexadecimal digits
                (?:(?<=::)|(?<!::):)    #   Colon unless preceeded by wildcard
            ){6}                        #
            (?:                         # Either
                [0-9a-f]{0,4}           #   Another group
                (?:(?<=::)|(?<!::):)    #   Colon unless preceeded by wildcard
                [0-9a-f]{0,4}           #   Last group
                (?: (?<=::)             #   Colon iff preceeded by exacly one colon
                 |  (?<!:)              #
                 |  (?<=:) (?<!::) :    #
                 )                      # OR
             |                          #   A v4 address with NO leading zeros
                (?:25[0-4]|2[0-4]\d|1\d\d|[1-9]?\d)
                (?: \.
                    (?:25[0-4]|2[0-4]\d|1\d\d|[1-9]?\d)
                ){3}
            )
            \s*                         # Trailing whitespace
            $
        """, re.VERBOSE | re.IGNORECASE | re.DOTALL)
        return pattern.match(value) is not None

    @staticmethod
    def validate_ipv4(value, **kwargs):
        # S/0: question 319279
        pattern = re.compile(r"""
            ^
            (?:
              # Dotted variants:
              (?:
                # Decimal 1-255 (no leading 0's)
                [3-9]\d?|2(?:5[0-5]|[0-4]?\d)?|1\d{0,2}
              |
                0x0*[0-9a-f]{1,2}  # Hexadecimal 0x0 - 0xFF (possible leading 0's)
              |
                0+[1-3]?[0-7]{0,2} # Octal 0 - 0377 (possible leading 0's)
              )
              (?:                  # Repeat 0-3 times, separated by a dot
                \.
                (?:
                  [3-9]\d?|2(?:5[0-5]|[0-4]?\d)?|1\d{0,2}
                |
                  0x0*[0-9a-f]{1,2}
                |
                  0+[1-3]?[0-7]{0,2}
                )
              ){0,3}
            |
              0x0*[0-9a-f]{1,8}    # Hexadecimal notation, 0x0 - 0xffffffff
            |
              0+[0-3]?[0-7]{0,10}  # Octal notation, 0 - 037777777777
            |
              # Decimal notation, 1-4294967295:
              429496729[0-5]|42949672[0-8]\d|4294967[01]\d\d|429496[0-6]\d{3}|
              42949[0-5]\d{4}|4294[0-8]\d{5}|429[0-3]\d{6}|42[0-8]\d{7}|
              4[01]\d{8}|[1-3]\d{0,9}|[4-9]\d{0,8}
            )
            $
        """, re.VERBOSE | re.IGNORECASE)
        return pattern.match(value) is not None

    def validate_json(self, value, **kwargs):
        return self._can_call_with_method(json.loads, value)

    def validate_lt(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='lt')
        value = self._get_size(value)
        lower = self._get_size(self._attribute_value(params[0]))
        return value < lower

    def validate_lte(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='lte')
        value = self._get_size(value)
        lower = self._get_size(self._attribute_value(params[0]))
        return value <= lower

    def validate_max(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='max')
        value = self._get_size(value)
        upper = self._get_size(params[0])
        return value <= upper

    def validate_mime_types(self, value, params, **kwargs):
        if not self.validate_file(value):
            return False
        self._assert_params_size(size=1, params=params, rule='mime_types')
        return value.mimetype in params

    def validate_min(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='min')
        value = self._get_size(value)
        lower = self._get_size(params[0])
        return value >= lower

    def validate_not_in(self, value, params, **kwargs):
        return not self.validate_in(value, params)

    def validate_not_regex(self, value, params, **kwargs):
        return not self.validate_regex(value, params)

    @staticmethod
    def validate_nullable(value, **kwargs):
        return True

    def validate_numeric(self, value, **kwargs):
        return self._can_call_with_method(float, value)

    def validate_present(self, attribute, **kwargs):
        accessors = attribute.split('.')
        request_param = self._request
        for accessor in accessors:
            if accessor not in request_param:
                return False
            request_param = request_param[accessor]
        return True

    def validate_regex(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='regex')
        self._assert_with_method(re.compile, params[0])
        return re.match(params[0], value)

    def validate_required(self, value, attribute, nullable, **kwargs):
        if not value and not nullable:
            return False
        return self.validate_present(attribute)

    def validate_required_if(self, value, attribute, params, nullable, **kwargs):
        self._assert_params_size(size=2, params=params, rule='required_if')
        other_value = self._attribute_value(params[0])
        if str(other_value) in params[1:]:
            return self.validate_required(value, attribute, nullable)
        return True

    def validate_required_unless(self, value, attribute, params, nullable, **kwargs):
        self._assert_params_size(size=2, params=params, rule='required_unless')
        other_value = self._attribute_value(params[0])
        if other_value not in params[1:]:
            return self.validate_required(value, attribute, nullable)
        return True

    def validate_required_with(self, value, attribute, params, nullable, **kwargs):
        self._assert_params_size(size=1, params=params, rule='required_with')
        for param in params:
            if self.validate_present(param):
                return self.validate_required(value, attribute, nullable)
        return True

    def validate_required_with_all(self, value, attribute, params, nullable, **kwargs):
        self._assert_params_size(size=1, params=params,
                rule='required_with_all')
        for param in params:
            if not self.validate_present(param):
                return True
        return self.validate_required(value, attribute, nullable)

    def validate_required_without(self, value, attribute, params, nullable, **kwargs):
        self._assert_params_size(size=1, params=params,
                rule='required_without')
        for param in params:
            if not self.validate_present(param):
                return self.validate_required(value, attribute, nullable)
        return True

    def validate_required_without_all(self, value, attribute, params, nullable, **kwargs):
        self._assert_params_size(size=1, params=params,
                rule='required_without_all')
        for param in params:
            if self.validate_present(param):
                return True
        return self.validate_required(value, attribute, nullable)

    def validate_same(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='same')
        other_value = self._attribute_value(params[0])
        return value == other_value

    def validate_size(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='size')
        self._assert_with_method(float, params[0])
        other_value = params[0]
        if other_value.count('.') > 0:
            other_value = float(other_value)
        else:
            other_value = int(other_value)
        return self._get_size(value) == other_value

    def validate_starts_with(self, value, params, **kwargs):
        self._assert_params_size(size=1, params=params, rule='starts_with')
        return str(value).startswith(params[0])

    @staticmethod
    def validate_string(value, **kwargs):
        return isinstance(value,
                str if sys.version_info[0] >= 3 else basestring)

    @staticmethod
    def validate_timezone(value, **kwargs):
        return value in pytz.all_timezones

    @staticmethod
    def validate_unique(value, **kwargs):
        return False

    @staticmethod
    def validate_url(value, **kwargs):
        pattern = re.compile(r"""
            ^
            (https?|ftp)://  # http, https or ftp
            (?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.? # domain
                |
            localhost
                |
            \d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}) # ...or ip
            (?::\d+)?                           # optional port
            (?:/?|[/?]\S+)
            $
        """, re.VERBOSE | re.IGNORECASE | re.DOTALL)
        return pattern.match(value) is not None

    @staticmethod
    def validate_uuid(value, **kwargs):
        return re.match(
            r'^[\da-f]{8}-[\da-f]{4}-[\da-f]{4}-[\da-f]{4}-[\da-f]{12}$',
            str(value).lower()
        ) is not None

    @staticmethod
    def _compare_dates(first, second, comparator):
        try:
            return comparator(dateparse(first), dateparse(second))
        except Exception:
            return False

    def _can_call_with_method(self, method, value):
        try:
            self._assert_with_method(method, value)
            return True
        except ValueError:
            return False

    @staticmethod
    def _has_rule(rules, rule_name):
        for rule in rules:
            if rule['name'] == rule_name:
                return True
        return False

    def _get_rule_handler(self, rule_name):
        handler_name = 'validate_' + rule_name
        if handler_name in self._custom_handlers:
            return self._custom_handlers[handler_name]['handler']
        elif hasattr(self, handler_name):
            return getattr(self, handler_name)
        else:
            raise ValueError("Validator: no handler for rule " + rule_name)

    def _get_size(self, value, rules=None):
        rules = rules or {}
        value_type = self._get_type(value, rules)
        if value_type == 'array':
            return len(ast.literal_eval(str(value)))
        elif value_type == 'numeric':
            return float(value)
        elif value_type == 'file':
            value.seek(0, os.SEEK_END)
            return round(value.tell() /  1024.0, 0)
        return len(str(value))

    def _get_type(self, value, rules=None):
        rules = rules or {}
        value_type = self._get_type_from_rules(value, rules)
        if not value_type:
            value_type = self._get_type_from_value(value)
        return value_type

    def _get_type_from_rules(self, value, rules):
        array_rules = ['array']
        numeric_rules = ['integer', 'numeric']
        file_rules = ['file', 'image', 'dimensions']
        string_rules = ['alpha', 'alpha_dash', 'string']
        if self._has_any_of_rules(string_rules, rules):
            return 'string'
        elif self._has_any_of_rules(numeric_rules, rules) and self.validate_numeric(value):
            return 'numeric'
        elif self._has_any_of_rules(array_rules, rules) and self.validate_array(value):
            return 'array'
        elif self._has_any_of_rules(file_rules, rules) and self.validate_file(value):
            return 'file'

    def _get_type_from_value(self, value):
        if self.validate_numeric(value):
            return 'numeric'
        elif self.validate_array(value):
            return 'array'
        elif self.validate_file(value):
            return 'file'
        return 'string'

    def _has_any_of_rules(self, subset, rules):
        for rule in subset:
            if self._has_rule(rules, rule):
                return True
        return False

    def _attribute_value(self, attribute):
        accessors = attribute.split('.')
        request_param = self._request
        for accessor in accessors:
            if accessor not in request_param:
                return None
            request_param = request_param[accessor]
        return request_param

    @staticmethod
    def _assert_params_size(size, params, rule):
        if size > len(params):
            raise ValueError(
                '%s rule requires at least %s parameter(s), non provided.' %
                (rule.title(), size)
            )

    @staticmethod
    def _assert_with_method(method, value):
        try:
            method(value)
        except:
            raise ValueError(
                'Cannot call method %s with value %s' %
                (method.__name__, str(value))
            )




import unittest

from werkzeug.datastructures import MultiDict

from flask_sieve.requests import FormRequest, JsonRequest
from flask_sieve.exceptions import ValidationException

class FormMockRequest:
    def __init__(self, data=None):
        data = data or {}
        self.form = MultiDict([(k, v) for k, v in data.items()])
        self.files = MultiDict([])
        self.is_json = False


class JsonMockRequest:
    def __init__(self, data=None):
        self.json = data or {}
        self.is_json = True

class TestFormRequest(FormRequest):
    @staticmethod
    def rules():
        return {
            'name': ['required', 'string', 'min:6'],
            'email':['required', 'email'],
            'password': ['required', 'min:8', 'confirmed']
        }


class TestJsonRequest(JsonRequest):
    @staticmethod
    def rules():
        return {
            'name': ['required', 'string', 'min:6'],
            'email': ['required', 'email'],
            'password': ['required', 'min:8', 'confirmed']
        }


class TestRequests(unittest.TestCase):
    def setUp(self):
        self.valid_data = {
            'name': 'John Doe',
            'email': 'john@doe.com',
            'password': 'john_doe',
            'password_confirmation': 'john_doe',
        }
        self.invalid_data = {
            'name': 'John',
            'email': 'johndoe.com',
            'password': 'johh_doe',
        }

    def test_form_request_fails(self):
        request = FormMockRequest(self.invalid_data)
        form_request = TestFormRequest(request)
        with self.assertRaises(ValidationException):
            form_request.validate()

    def test_form_request_passes(self):
        request = FormMockRequest(self.valid_data)
        form_request = TestFormRequest(request)
        self.assertTrue(form_request.validate())

    def test_json_request_fails(self):
        request = JsonMockRequest(self.invalid_data)
        json_request = TestJsonRequest(request=request)
        with self.assertRaises(ValidationException):
            json_request.validate()

    def test_json_request_passes(self):
        request = JsonMockRequest(self.valid_data)
        json_request = TestJsonRequest(request=request)
        self.assertTrue(json_request.validate())

    def test_json_request_checks_is_json(self):
        request = FormMockRequest(self.invalid_data)
        with self.assertRaises(ValidationException):
            TestJsonRequest(request=request)



from .requests import JsonRequest, FormRequest
from .validator import validate, Validator
from .exceptions import ValidationException, register_error_handler

class Sieve:
    def __init__(self, app):
        register_error_handler(app)



from flask import jsonify

class ValidationException(Exception):
    def __init__(self, errors):
        self.errors = errors


def register_error_handler(app):
    def validations_error_handler(ex):
        return jsonify({
            'success': False,
            'message': 'Validation error',
            'errors': ex.errors
        }), 400
    app.register_error_handler(
        ValidationException,
        validations_error_handler
    )




import unittest

from flask_sieve.parser import Parser


class TestParser(unittest.TestCase):
    def setUp(self):
        self._parser = Parser()

    def test_parses_rules_correctly(self):
        rules = {
            'name': ['required', 'string', 'min:6'],
            'email': ['required', 'email', 'in:a@b.com,b@c.com'],
            'password': ['required', 'min:8', 'confirmed']
        }
        self._parser.set_rules(rules)
        parsed_rules = self._parser.parsed_rules()
        self.assertDictEqual(
            parsed_rules,
            {
                'name': [
                    {'name': 'required', 'params': []},
                    {'name': 'string', 'params': []},
                    {'name': 'min', 'params': ['6']}
                ],
                'email': [
                    {'name': 'required', 'params': []},
                    {'name': 'email', 'params': []},
                    {'name': 'in', 'params': ['a@b.com', 'b@c.com']},
                ],
                'password': [
                    {'name': 'required', 'params': []},
                    {'name': 'min', 'params': ['8']},
                    {'name': 'confirmed', 'params': []},
                ]

            }
        )



import re

from flask_sieve.lang.en import rule_messages


class Translator:
    def __init__(self, validations=None, custom_messages=None,
            handler_messages=None):
        self._validations = validations or {}
        self._custom_messages = custom_messages or {}
        self._handler_messages = handler_messages or {}
        self._size_rules = ['between', 'gt', 'gte', 'lt', 'lte', 'max', 'min', 'size']

    def set_custom_messages(self, messages):
        self._custom_messages = messages

    def set_handler_messages(self, messages):
        self._handler_messages = messages

    def set_validations(self, validations):
        self._validations = validations

    def translated_errors(self):
        error_messages = {}
        for attribute, validations in self._validations.items():
            translated = []
            for validation in validations:
                if not validation['is_valid']:
                    custom_message_key = validation['attribute'] + '.' + validation['rule']
                    if custom_message_key in self._custom_messages:
                        translated.append(self._custom_messages[custom_message_key])
                    else:
                        translated.append(self._translate_validation(validation))
            if len(translated):
                error_messages[attribute] = translated
        return error_messages

    def _translate_validation(self, validation):
        messages = self._get_all_messages()
        message = messages.get(validation['rule'])
        if validation['rule'] in self._size_rules:
            message = message[validation['attribute_type']]
        message_fields = self._extract_message_fields(message)
        fields_to_params = self._zip_fields_to_params(
            fields=message_fields,
            params=validation['params']
        )
        for field in message_fields:
            if field == ':attribute':
                message = message.replace(field, validation['attribute'])
            else:
                message = message.replace(field, fields_to_params[field])
        return message

    def _get_all_messages(self):
        messages = {}
        messages.update(rule_messages)
        messages.update(self._handler_messages)
        return messages

    @staticmethod
    def _extract_message_fields(message):
        return re.findall(':\w+', message)

    @staticmethod
    def _zip_fields_to_params(params, fields):
        zipped = {}
        for field in fields:
            if field == ':attribute':
                continue
            _, param_index = field.split('_')
            if param_index == 'all':
                zipped[field] = ', '.join(params)
            elif param_index == 'rest':
                zipped[field] = ', '.join(params[1:])
            else:
                index = int(param_index)
                zipped[field] = params[index]
        return zipped



import json
import unittest

from werkzeug.datastructures import FileStorage

from flask_sieve.parser import Parser
from flask_sieve.rules_processor import RulesProcessor


class TestRulesProcessor(unittest.TestCase):
    def setUp(self):
        self.parser = Parser()
        self.processor = RulesProcessor()
        self.stream = open('tests/files/image.png')
        self.image_file = FileStorage(
            stream=self.stream,
            filename='tests/files/image.png',
            content_type='image/png',
            name='image'
        )
        self.invalid_file = FileStorage(
            stream=self.stream,
            filename='invalid.png'
        )

    def test_validates_accepted(self):
        acceptables = [ 1, '1', 'true', 'yes', 'on', True]
        for acceptable in acceptables:
            self.assert_passes(
                rules={'field': ['accepted']},
                request={'field': acceptable},
            )
        self.assert_fails(
            rules={'field': ['accepted']},
            request={'field': 'hi'},
        )

    def test_validates_active_url(self):
        self.assert_passes(
            rules={'field': ['active_url']},
            request={'field': 'https://google.com'},
        )
        self.assert_fails(
            rules={'field': ['bail', 'active_url']},
            request={'field': 'https://gxxgle.cxm'},
        )

    def test_validates_after(self):
        self.assert_passes(
            rules={'field': ['after:2018-08-02']},
            request={'field': '2nd Sept 2018'}
        )
        self.assert_fails(
            rules={'field': ['after:2018-08-03']},
            request={'field': '3rd Aug 2018'}
        )
        self.assert_fails(
            rules={'field': ['after:2018-09-02']},
            request={'field': '2nd Sept 2017'}
        )

    def test_validates_after_or_equal(self):
        self.assert_passes(
            rules={'field': ['after_or_equal:3/8/2018']},
            request={'field': '3rd Sept 2018'}
        )
        self.assert_passes(
            rules={'field': ['after_or_equal:3/8/2018']},
            request={'field': '3rd Aug 2018'}
        )
        self.assert_fails(
            rules={'field': ['after_or_equal:5/8/2018']},
            request={'field': '5th Sept 2017'}
        )

    def test_validates_alpha(self):
        self.assert_passes(
            rules={'field': ['alpha']},
            request={'field': 'hi there'}
        )
        self.assert_fails(
            rules={'field': ['alpha']},
            request={}
        )
        self.assert_fails(
            rules={'field': ['alpha']},
            request={'field': 'abcd1234'}
        )

    def test_validates_alpha_dash(self):
        self.assert_passes(
            rules={'field': ['alpha_dash']},
            request={'field': 'hi-there__ friend'}
        )
        self.assert_fails(
            rules={'field': ['alpha_dash']},
            request={}
        )
        self.assert_fails(
            rules={'field': ['alpha_dash']},
            request={'field': 'abcd1234'}
        )

    def test_validates_alpha_num(self):
        self.assert_passes(
            rules={'field': ['alpha_num']},
            request={'field': 'abcd1234'}
        )
        self.assert_fails(
            rules={'field': ['alpha_num']},
            request={}
        )
        self.assert_fails(
            rules={'field': ['alpha_num']},
            request={'field': 'hi-there__ friend'}
        )

    def test_validates_array(self):
        self.assert_passes(
            rules={'field': ['array']},
            request={'field': '[1, 2, 3]'}
        )
        self.assert_fails(
            rules={'field': ['array']},
            request={'field': '[1, 2, 3}'}
        )

    def test_validates_before(self):
        self.assert_passes(
            rules={'field': ['before:2018-09-02']},
            request={'field': '2nd Nov 2017'}
        )
        self.assert_fails(
            rules={'field': ['before:2018-09-02']},
            request={'field': '2nd Oct 2018'}
        )
        self.assert_fails(
            rules={'field': ['before:2018-08-02']},
            request={'field': '2nd Sept 2018'}
        )

    def test_validates_before_or_equal(self):
        self.assert_passes(
            rules={'field': ['before_or_equal:2018-09-03']},
            request={'field': '3rd Nov 2017'}
        )
        self.assert_passes(
            rules={'field': ['before_or_equal:2018-09-03']},
            request={'field': '3rd Sept 2018'}
        )
        self.assert_fails(
            rules={'field': ['before_or_equal:2018-08-02']},
            request={'field': '2nd Sept 2018'}
        )

    def test_validates_between(self):
        self.assert_passes(
            rules={'field': ['between:2.0,50']},
            request={'field': '3'}
        )
        self.assert_passes(
            rules={'field': ['between:0.002,5']},
            request={'field': '5'}
        )
        self.assert_fails(
            rules={'field': ['between:2.0,5']},
            request={'field': '30'}
        )

    def test_validates_boolean(self):
        for value in [True, False, 1, 0, '1', '0']:
            self.assert_passes(
                rules={'field': ['boolean']},
                request={'field': value}
            )

        self.assert_fails(
            rules={'field': ['boolean']},
            request={'field': 'hi'}
        )


    def test_validates_confirmed(self):
        self.assert_passes(
            rules={'field': ['confirmed']},
            request={'field': 1, 'field_confirmation': 1}
        )
        self.assert_fails(
            rules={'field': ['confirmed']},
            request={'field': 2, 'field_confirmation': 1}
        )
        self.assert_fails(
            rules={'field': ['confirmed']},
            request={'field': 2}
        )

    def test_validates_date(self):
        self.assert_passes(
            rules={'field': ['date']},
            request={'field': '2018-01-10'}
        )
        self.assert_passes(
            rules={'field': ['date']},
            request={'field': '3rd August 2018'}
        )
        self.assert_fails(
            rules={'field': ['date']},
            request={'field': 'now'}
        )
        self.assert_fails(
            rules={'field': ['date']},
            request={'field': True}
        )

    def test_validates_date_equals(self):
        self.assert_passes(
            rules={'field': ['date_equals:2018-08-02']},
            request={'field': '2nd August 2018'}
        )
        self.assert_fails(
            rules={'field': ['date_equals:2018-08-03']},
            request={'field': '3rd March 2019'}
        )
        self.assert_fails(
            rules={'field': ['date_equals:2018-09-02']},
            request={'field': '2nd Sept 2017'}
        )

    def test_validates_different(self):
        self.assert_passes(
            rules={'field': ['different:field_2']},
            request={'field': 1, 'field_2': 2}
        )
        self.assert_fails(
            rules={'field': ['different:field_2']},
            request={'field': 1, 'field_2': 1}
        )

    def test_validates_digits(self):
        self.assert_passes(
            rules={'field': ['digits:1']},
            request={'field': 1}
        )
        self.assert_passes(
            rules={'field': ['digits:4']},
            request={'field': '1.243'}
        )
        self.assert_passes(
            rules={'field': ['digits:4']},
            request={'field': 0.243}
        )
        self.assert_fails(
            rules={'field': ['digits:1']},
            request={'field': '123ab'}
        )

    def test_validates_digits_between(self):
        self.assert_passes(
            rules={'field': ['digits_between:1,3']},
            request={'field': 1}
        )
        self.assert_passes(
            rules={'field': ['digits_between:1,3']},
            request={'field': '1.24'}
        )
        self.assert_passes(
            rules={'field': ['digits_between:0,4']},
            request={'field': 0.243}
        )
        self.assert_fails(
            rules={'field': ['digits_between:0,3']},
            request={'field': '3.142'}
        )

    def test_validates_dimensions(self):
        self.assert_passes(
            rules={'field': ['dimensions:1x1']},
            request={'field': self.image_file}
        )
        self.assert_fails(
            rules={'field': ['dimensions:2x1']},
            request={'field': self.image_file}
        )
        self.assert_fails(
            rules={'field': ['dimensions:2x1']},
            request={'field': self.invalid_file}
        )
        self.assert_fails(
            rules={'field': ['dimensions:2x1']},
            request={'field': 'hi'}
        )

    def test_validates_distinct(self):
        self.assert_passes(
            rules={'field': ['distinct']},
            request={'field': [1, 2, 3]}
        )
        self.assert_passes(
            rules={'field': ['distinct']},
            request={'field': ['hi', 'there', 'friend']}
        )
        self.assert_passes(
            rules={'field': ['distinct']},
            request={'field': '[1, 2, 4]'}
        )
        self.assert_fails(
            rules={'field': ['distinct']},
            request={'field': '["a":1, "b":2, "c":4}'}
        )
        self.assert_fails(
            rules={'field': ['distinct']},
            request={'field': '{"a":1, "b":2, "c":4}'}
        )
        self.assert_fails(
            rules={'field': ['distinct']},
            request={'field': [1, 1]}
        )

    def test_validates_exists(self):
        self.assert_fails(
            rules={'field': ['exists']},
            request={}
        )

    def test_validates_extension(self):
        self.assert_passes(
            rules={'field': ['extension:png']},
            request={'field': self.image_file}
        )
        self.assert_fails(
            rules={'field': ['extension:png']},
            request={'field': 1}
        )

    def test_validates_email(self):
        self.assert_passes(
            rules={'field': ['email', 'nullable']},
            request={'field': 'john@doe.com'}
        )
        self.assert_passes(
            rules={'field': ['email']},
            request={'field': 'jane@doe.com'}
        )
        self.assert_fails(
            rules={'field': ['email']},
            request={'field': 'hi there'}
        )

    def test_validates_file(self):
        self.assert_passes(
            rules={'field': ['file']},
            request={'field': self.image_file}
        )
        self.assert_fails(
            rules={'field': ['file']},
            request={'field': 'hi'}
        )

    def test_validates_filled(self):
        self.assert_passes(
            rules={'field': ['filled']},
            request={'field': 'hi'}
        )
        self.assert_passes(
            rules={'field': ['filled']},
            request={}
        )
        self.assert_fails(
            rules={'field': ['filled']},
            request={'field': None}
        )

    def test_validates_gt(self):
        self.assert_passes(
            rules={'field': ['gt:field_2']},
            request={'field': -1, 'field_2': -10}
        )
        self.assert_fails(
            rules={'field': ['gt:field_2']},
            request={'field': 1, 'field_2': 1}
        )
        self.assert_fails(
            rules={'field': ['gt:field_2']},
            request={'field': 1, 'field_2': 10}
        )

    def test_validates_gte(self):
        self.assert_passes(
            rules={'field': ['gte:field_2']},
            request={'field': -1, 'field_2': -10}
        )
        self.assert_passes(
            rules={'field': ['gte:field_2']},
            request={'field': 1, 'field_2': 1}
        )
        self.assert_fails(
            rules={'field': ['gte:field_2']},
            request={'field': 1, 'field_2': 10}
        )

    def test_validates_image(self):
        self.assert_passes(
            rules={'field': ['image']},
            request={'field': self.image_file}
        )
        self.assert_fails(
            rules={'field': ['image']},
            request={'field': 1}
        )
        self.assert_fails(
            rules={'field': ['image']},
            request={}
        )

    def test_validates_in(self):
        self.assert_passes(
            rules={'field': ['in:male,female']},
            request={'field': 'male'}
        )
        self.assert_fails(
            rules={'field': ['in:male,female']},
            request={'field': 'person'}
        )

    def test_validates_in_array(self):
        self.assert_passes(
            rules={'field': ['in_array:field_2']},
            request={'field': 'male', 'field_2': ['male', 'female']}
        )
        self.assert_fails(
            rules={'field': ['in_array:field_2']},
            request={'field': '{"a":1, "b":2, "c":4}', 'field_2':'hi'}
        )
        self.assert_fails(
            rules={'field': ['in_array:field_2']},
            request={'field': 'female', 'field_2': []}
        )

    def test_validates_integer(self):
        self.assert_passes(
            rules={'field': ['integer']},
            request={'field': 100}
        )
        self.assert_passes(
            rules={'field': ['integer']},
            request={'field': '100'}
        )
        self.assert_fails(
            rules={'field': ['integer']},
            request={'field': '1.200'}
        )
        self.assert_fails(
            rules={'field': ['integer']},
            request={'field': 1.204}
        )

    def test_validates_ip(self):
        self.assert_passes(
            rules={'field': ['ip']},
            request={'field': '0.1.0.0'}
        )
        self.assert_passes(
            rules={'field': ['ip']},
            request={'field': '127.0.0.1'}
        )
        self.assert_passes(
            rules={'field': ['ip']},
            request={'field': '2001:0db8:0000:0000:0000:ff00:0042:8329'}
        )
        self.assert_fails(
            rules={'field': ['ip']},
            request={'field': '127.d.0.1'}
        )
        self.assert_fails(
            rules={'field': ['ip']},
            request={'field': '20:0db8:0000:0000:0000:xx:0042:8329'}
        )

    def test_validates_ipv4(self):
        self.assert_passes(
            rules={'field': ['ipv4']},
            request={'field': '0.1.0.0'}
        )
        self.assert_passes(
            rules={'field': ['ipv4']},
            request={'field': '127.0.0.1'}
        )
        self.assert_fails(
            rules={'field': ['ipv4']},
            request={'field': '127.d.0.1'}
        )
        self.assert_fails(
            rules={'field': ['ipv4']},
            request={'field': '2001:0db8:0000:0000:0000:ff00:0042:8329'}
        )

    def test_validates_ipv6(self):
        self.assert_passes(
            rules={'field': ['ipv6']},
            request={'field': '2001:0db8:0000:0000:0000:ff00:0042:8329'}
        )
        self.assert_fails(
            rules={'field': ['ipv6']},
            request={'field': '127.0.0.1'}
        )
        self.assert_fails(
            rules={'field': ['ipv6']},
            request={'field': '20:0db8:0000:0000:0000:xx:0042:8329'}
        )

    def test_validates_json(self):
        self.assert_passes(
            rules={'field': ['json']},
            request={'field': json.dumps({'hi': 'there'})}
        )
        self.assert_fails(
            rules={'field': ['json']},
            request={'field': 'hello'}
        )
        self.assert_fails(
            rules={'field': ['json']},
            request={'field': 1}
        )

    def test_validates_lt(self):
        self.assert_passes(
            rules={'field': ['lt:field_2']},
            request={'field': -10, 'field_2': -1}
        )
        self.assert_fails(
            rules={'field': ['lt:field_2']},
            request={'field': 1, 'field_2': 1}
        )
        self.assert_fails(
            rules={'field': ['lt:field_2']},
            request={'field': 10, 'field_2': 1}
        )

    def test_validates_lte(self):
        self.assert_passes(
            rules={'field': ['lte:field_2']},
            request={'field': -10, 'field_2': -1}
        )
        self.assert_passes(
            rules={'field': ['lte:field_2']},
            request={'field': 1, 'field_2': 1}
        )
        self.assert_fails(
            rules={'field': ['lte:field_2']},
            request={'field': 10, 'field_2': 1}
        )

    def test_validates_max(self):
        self.assert_passes(
            rules={'field': ['max:20']},
            request={'field': 10}
        )
        self.assert_fails(
            rules={'field': ['max:10']},
            request={'field': 30}
        )

    def test_validates_mime_types(self):
        self.assert_passes(
            rules={'field': ['mime_types:image/png']},
            request={'field': self.image_file}
        )
        self.assert_fails(
            rules={'field': ['mime_types:image/png']},
            request={'field': 1}
        )

    def test_validates_min(self):
        self.assert_passes(
            rules={'field': ['min:0']},
            request={'field': 10}
        )
        self.assert_fails(
            rules={'field': ['min:10']},
            request={'field': 0}
        )

    def test_validates_not_in(self):
        self.assert_passes(
            rules={'field': ['not_in:male,female']},
            request={'field': 'person'}
        )
        self.assert_fails(
            rules={'field': ['not_in:male,female']},
            request={'field': 'female'}
        )

    def test_validates_not_regex(self):
        self.assert_passes(
            rules={'field': ['not_regex:[0-9]\|']},
            request={'field': 'person\|'}
        )
        self.assert_passes(
            rules={'field': ['not_regex:\d']},
            request={'field': 'person'}
        )
        self.assert_fails(
            rules={'field': ['not_regex:\d']},
            request={'field': '10'}
        )

    def test_validates_numeric(self):
        self.assert_passes(
            rules={'field': ['numeric']},
            request={'field': '1.2004'}
        )
        self.assert_passes(
            rules={'field': ['numeric']},
            request={'field': 2000.04}
        )
        self.assert_fails(
            rules={'field': ['numeric']},
            request={'field': 'x10'}
        )

    def test_validates_present(self):
        self.assert_passes(
            rules={'field': ['present']},
            request={'field': '1.2004'}
        )
        self.assert_passes(
            rules={'field': ['present']},
            request={'field': 2000.04}
        )
        self.assert_passes(
            rules={'field': ['present']},
            request={'field': None}
        )
        self.assert_fails(
            rules={'field': ['present']},
            request={'field_2': 'x10'}
        )

    def test_validates_regex(self):
        self.assert_passes(
            rules={'field': ['regex:[0-9]']},
            request={'field': '01234567'}
        )
        self.assert_passes(
            rules={'field': ['regex:^\w']},
            request={'field': 'person'}
        )
        self.assert_fails(
            rules={'field': ['regex:\d']},
            request={'field': 'hi there'}
        )

    def test_validates_required(self):
        self.assert_passes(
            rules={'field': ['required']},
            request={'field': 'hi'}
        )
        self.assert_fails(
            rules={'field': ['required']},
            request={'field': []}
        )
        self.assert_fails(
            rules={'field': ['required']},
            request={'field': ''}
        )
        self.assert_fails(
            rules={'field': ['required']},
            request={'field': None}
        )

    def test_validates_required_if(self):
        self.assert_passes(
            rules={'field': ['required_if:field_2,one,two']},
            request={'field': 'three', 'field_2': 'one'}
        )
        self.assert_passes(
            rules={'field': ['required_if:field_2,one,two']},
            request={'field': '', 'field_2': 'three'}
        )
        self.assert_passes(
            rules={'field': ['size:0']},
            request={'field': self.image_file}
        )
        self.assert_fails(
            rules={'field': ['required_if:field_2,one,two']},
            request={'field': '', 'field_2': 'one'}
        )

    def test_validates_required_unless(self):
        self.assert_passes(
            rules={'field': ['required_unless:field_2,one,two']},
            request={'field': 'three', 'field_2': 'four'}
        )
        self.assert_passes(
            rules={'field': ['required_unless:field_2,one,two']},
            request={'field': '', 'field_2': 'one'}
        )
        self.assert_fails(
            rules={'field': ['required_unless:field_2,one,two']},
            request={'field': '', 'field_2': 'three'}
        )

    def test_validates_required_with(self):
        self.assert_passes(
            rules={'field': ['required_with:field_2']},
            request={'field': 'three', 'field_2': 'four'}
        )
        self.assert_passes(
            rules={'field': ['required_with:field_2']},
            request={'field': ''}
        )
        self.assert_passes(
            rules={'field': ['required_with:field_2']},
            request={'field': 'hi', 'field_2': ''}
        )
        self.assert_fails(
            rules={'field': ['required_with:field_2']},
            request={'field': '', 'field_2': ''}
        )

    def test_validates_required_with_all(self):
        self.assert_passes(
            rules={'field': ['required_with_all:field_2,field_3']},
            request={'field': 'three', 'field_2': 'four', 'field_3': 'five'}
        )
        self.assert_passes(
            rules={'field': ['required_with_all:field_2,field_3']},
            request={'field': '', 'field_2': 'hi'}
        )
        self.assert_fails(
            rules={'field': ['required_with_all:field_2']},
            request={'field': '', 'field_2': ''}
        )

    def test_validates_required_without(self):
        self.assert_passes(
            rules={'field': ['required_without:field_2,field_3']},
            request={'field': '', 'field_2': 'four', 'field_3': 'five'}
        )
        self.assert_passes(
            rules={'field': ['required_without:field_2,field_3']},
            request={'field': 'hello', 'field_2': 'hi'}
        )
        self.assert_fails(
            rules={'field': ['required_without:field_2,field_3']},
            request={'field': '', 'field_2': ''}
        )

    def test_validates_required_without_all(self):
        self.assert_passes(
            rules={'field': ['required_without_all:field_2,field_3']},
            request={'field': '', 'field_2': 'four'}
        )
        self.assert_passes(
            rules={'field': ['required_without_all:field_2,field_3']},
            request={'field': '', 'field_2': 'hi', 'field_3': 'there'}
        )
        self.assert_passes(
            rules={'field': ['required_without_all:field_2,field_3']},
            request={'field': 'hi' }
        )
        self.assert_fails(
            rules={'field': ['required_without_all:field_2,field_3']},
            request={'field': '' }
        )

    def test_validates_same(self):
        self.assert_passes(
            rules={'field': ['same:field_2']},
            request={'field': 1, 'field_2': 1}
        )
        self.assert_fails(
            rules={'field': ['same:field_2']},
            request={'field': 1, 'field_2': 2}
        )

    def test_validates_size(self):
        self.assert_passes(
            rules={'field': ['size:2']},
            request={'field': 'hi'}
        )
        self.assert_passes(
            rules={'field': ['size:20']},
            request={'field': 20}
        )
        self.assert_passes(
            rules={'field': ['size:20.5']},
            request={'field': '20.5'}
        )
        self.assert_passes(
            rules={'field': ['array', 'size:2']},
            request={'field': '[1, 2]'}
        )
        self.assert_fails(
            rules={'field': ['size:1']},
            request={'field': 'hi'}
        )

    def test_validates_starts_with(self):
        self.assert_passes(
            rules={'field': ['starts_with:hi']},
            request={'field': 'hi there'}
        )
        self.assert_passes(
            rules={'field': ['starts_with:2']},
            request={'field': 20}
        )
        self.assert_fails(
            rules={'field': ['starts_with:hi']},
            request={'field': '20.5'}
        )

    def test_validates_string(self):
        self.assert_passes(
            rules={'field': ['string']},
            request={'field': 'hi there'}
        )
        self.assert_passes(
            rules={'field': ['string']},
            request={'field': '20.5'}
        )
        self.assert_fails(
            rules={'field': ['string']},
            request={'field': 20}
        )

    def test_validates_timezone(self):
        self.assert_passes(
            rules={'field': ['timezone']},
            request={'field': 'Africa/Nairobi'}
        )
        self.assert_passes(
            rules={'field': ['timezone']},
            request={'field': 'Atlantic/Cape_Verde'}
        )
        self.assert_passes(
            rules={'field': ['timezone']},
            request={'field': 'Etc/Universal'}
        )
        self.assert_fails(
            rules={'field': ['timezone']},
            request={'field': 'hi'}
        )

    def test_validates_unique(self):
        self.assert_fails(
            rules={'field': ['unique']},
            request={}
        )

    def test_validates_url(self):
        self.assert_passes(
            rules={'field': ['url']},
            request={'field': 'https://google.com'}
        )
        self.assert_passes(
            rules={'field': ['url']},
            request={'field': 'http://127.0.0.1:8080'}
        )
        self.assert_passes(
            rules={'field': ['url']},
            request={'field': 'ftp://127.0.0.1:8080'}
        )
        self.assert_fails(
            rules={'field': ['url']},
            request={'field': 'hi'}
        )

    def test_validates_uuid(self):
        self.assert_passes(
            rules={'field': ['uuid']},
            request={'field': '06d007b0-5608-11e9-8647-d663bd873d93'}
        )
        self.assert_fails(
            rules={'field': ['uuid']},
            request={'field': '06d007b0-560647-d663bd873d93'}
        )

    def test_custom_handlers(self):
        def validate_odd(value, **kwargs):
            return value % 2
        self.processor.register_rule_handler(
            handler=validate_odd,
            message='Not odd'
        )
        self.assert_passes(
            rules={'field': ['odd']},
            request={'field': 3}
        )
        self.assert_fails(
            rules={'field': ['odd']},
            request={'field': 2}
        )

    def test_get_rule_handler(self):
        handler = self.processor._get_rule_handler('ip')
        self.assertEqual(handler.__name__, 'validate_ip')

        with self.assertRaises(ValueError):
            self.processor._get_rule_handler('wow')

    def test_assert_params_size(self):
        handler = self.processor._get_rule_handler('ip')
        self.assertEqual(handler.__name__, 'validate_ip')

        with self.assertRaises(ValueError):
            self.processor._assert_params_size(size=1, params=[], rule='hi')

    def test_compare_dates(self):
        import operator
        self.assertTrue(
            self.processor._compare_dates(
                '2018-02-10',
                '2019-02-10',
                operator.lt
            )
        )
        self.assertFalse(
            self.processor._compare_dates(
                'S',
                '2019-02-10',
                operator.lt
            )
        )

    def assert_fails(self, rules, request):
        self._assert(rules, request, False)

    def assert_passes(self, rules, request):
        self._assert(rules, request, True)

    def _assert(self, rules, request, passes):
        p = self.parser
        v = self.processor
        v.set_request(request)
        p.set_rules(rules)
        v.set_rules(p.parsed_rules())
        self.assertTrue(v.passes() if passes else v.fails())

    def tearDown(self):
        self.stream.close()

if __name__ == '__main__':
    unittest.main()



from setuptools import setup, find_packages


setup(
    name='flask-sieve',
    description='A Laravel inspired requests validator for Flask',
    long_description='Find the documentation at https://flask-sieve.readthedocs.io/en/latest/',
    version='1.0.4',
    url='https://github.com/codingedward/flask-sieve',
    license='BSD-2',
    author='Edward Njoroge',
    author_email='codingedward@gmail.com',
    maintainer='Edward Njoroge',
    maintainer_email='codingedward@gmail.com',
    install_requires=[
        'Flask',
        'Pillow',
        'python-dateutil',
        'pytz',
        'requests',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    packages=find_packages(),
    project_urls={
        'Funding': 'https://donate.pypi.org',
        'Source': 'https://github.com/codingedward/flask-sieve/',
        'Tracker': 'https://github.com/codingedward/flask-sieve/issues',
    },
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',
    zip_safe=False
)



rule_messages = {
    'accepted': 'The :attribute must be accepted.',
    'active_url': 'The :attribute is not a valid URL.',
    'after': 'The :attribute must be a date after :date_0.',
    'after_or_equal': 'The :attribute must be a date after or equal to :date_0.',
    'alpha': 'The :attribute may only contain letters.',
    'alpha_dash': 'The :attribute may only contain letters, numbers, dashes and underscores.',
    'alpha_num': 'The :attribute may only contain letters and numbers.',
    'array': 'The :attribute must be an array.',
    'before': 'The :attribute must be a date before :date_0.',
    'before_or_equal': 'The :attribute must be a date before or equal to :date_0.',
    'between': {
        'numeric': 'The :attribute must be between :min_0 and :max_1.',
        'file': 'The :attribute must be between :min_0 and :max_1 kilobytes.',
        'string': 'The :attribute must be between :min_0 and :max_1 characters.',
        'array': 'The :attribute must have between :min_0 and :max_1 items.',
    },
    'boolean': 'The :attribute field must be true or false.',
    'confirmed': 'The :attribute confirmation does not match.',
    'date': 'The :attribute is not a valid date.',
    'date_equals': 'The :attribute must be a date equal to :date_0.',
    'date_format': 'The :attribute does not match the format :format_0.',
    'different': 'The :attribute and :other_0 must be different.',
    'digits': 'The :attribute must be :digits_0 digits.',
    'digits_between': 'The :attribute must be between :min_0 and :max_1 digits.',
    'dimensions': 'The :attribute has invalid image dimensions.',
    'distinct': 'The :attribute field has a duplicate value.',
    'email': 'The :attribute must be a valid email address.',
    'exists': 'The selected :attribute is invalid.',
    'file': 'The :attribute must be a file.',
    'filled': 'The :attribute field must have a value.',
    'gt': {
        'numeric': 'The :attribute must be greater than :value_0.',
        'file': 'The :attribute must be greater than :value_0 kilobytes.',
        'string': 'The :attribute must be greater than :value_0 characters.',
        'array': 'The :attribute must have more than :value_0 items.',
    },
    'gte': {
        'numeric': 'The :attribute must be greater than or equal :value_0.',
        'file': 'The :attribute must be greater than or equal :value_0 kilobytes.',
        'string': 'The :attribute must be greater than or equal :value_0 characters.',
        'array': 'The :attribute must have :value_0 items or more.',
    },
    'image': 'The :attribute must be an image.',
    'in': 'The selected :attribute is invalid.',
    'in_array': 'The :attribute field does not exist in :other_0.',
    'integer': 'The :attribute must be an integer.',
    'ip': 'The :attribute must be a valid IP address.',
    'ipv4': 'The :attribute must be a valid IPv4 address.',
    'ipv6': 'The :attribute must be a valid IPv6 address.',
    'json': 'The :attribute must be a valid JSON string.',
    'lt': {
        'numeric': 'The :attribute must be less than :value_0.',
        'file': 'The :attribute must be less than :value_0 kilobytes.',
        'string': 'The :attribute must be less than :value_0 characters.',
        'array': 'The :attribute must have less than :value_0 items.',
    },
    'lte': {
        'numeric': 'The :attribute must be less than or equal :value_0.',
        'file': 'The :attribute must be less than or equal :value_0 kilobytes.',
        'string': 'The :attribute must be less than or equal :value_0 characters.',
        'array': 'The :attribute must not have more than :value_0 items.',
    },
    'max': {
        'numeric': 'The :attribute may not be greater than :max_0.',
        'file': 'The :attribute may not be greater than :max_0 kilobytes.',
        'string': 'The :attribute may not be greater than :max_0 characters.',
        'array': 'The :attribute may not have more than :max_0 items.',
    },
    'mime_types': 'The :attribute must be a file of type: :values_0.',
    'min': {
        'numeric': 'The :attribute must be at least :min_0.',
        'file': 'The :attribute must be at least :min_0 kilobytes.',
        'string': 'The :attribute must be at least :min_0 characters.',
        'array': 'The :attribute must have at least :min_0 items.',
    },
    'not_in': 'The selected :attribute is invalid.',
    'not_regex': 'The :attribute format is invalid.',
    'numeric': 'The :attribute must be a number.',
    'present': 'The :attribute field must be present.',
    'regex': 'The :attribute format is invalid.',
    'required': 'The :attribute field is required.',
    'required_if': 'The :attribute field is required when :other_0 is :value_1.',
    'required_unless': 'The :attribute field is required unless :other_0 is in :values_rest.',
    'required_with': 'The :attribute field is required when :values_all is present.',
    'required_with_all': 'The :attribute field is required when :values_all are present.',
    'required_without': 'The :attribute field is required when :values_all is not present.',
    'required_without_all': 'The :attribute field is required when none of :values_all are present.',
    'same': 'The :attribute and :other_0 must match.',
    'size': {
        'numeric': 'The :attribute must be :size_0.',
        'file': 'The :attribute must be :size_0 kilobytes.',
        'string': 'The :attribute must be :size_0 characters.',
        'array': 'The :attribute must contain :size_0 items.',
    },
    'starts_with': 'The :attribute must start with one of the following: :values_all',
    'string': 'The :attribute must be a string.',
    'timezone': 'The :attribute must be a valid zone.',
    'unique': 'The :attribute has already been taken.',
    'uploaded': 'The :attribute failed to upload.',
    'url': 'The :attribute format is invalid.',
    'uuid': 'The :attribute must be a valid UUID.',
}
